<?php

namespace UnicaenCode\Generator;

use UnicaenCode\Service\CodeGeneratorService;
use UnicaenCode\Util;

class Controller implements GeneratorInterface
{
    /**
     * @param array $params
     *
     * Paramètres en entrée :
     *
     * - class => string nom de la classe pour laquelle le AwareTrait sera généré
     *
     * @return array
     */
    public function generate(array $params): array
    {
        $class    = $params['class'] ?? null;
        $type     = $params['type'] ?? Util::classType($class) ?? 'Controller';
        $author   = $params['author'] ?? Util::getAuthor();
        $template = $params['template'] ?? 'Controller';
        $subDir   = $params['subDir'] ?? false;

        $namespace = Util::classNamespace($class);
        $classname = Util::classClassname($class);
        $filename  = Util::classFilename($class);

        $params['namespace'] = $namespace;
        $params['classname'] = $classname;
        $params['author']    = $author;
        $params['template']  = $template;
        $params['type']      = $type;
        $params['filename']  = $filename;

        $params['controllerConfig'] = Util::codeGenerator()->transferParams($params, ['route', 'class', 'type', 'author', 'subDir']);
        $params['controllerConfig']['generator'] = ControllerConfig::class;

        Util::codeGenerator()->generateFrom($params, 'factory');

        if (isset($params['factory']['class'])) {
            $params['controllerConfig']['factoryClass'] = Util::classTruncated($params['factory']['class'], Util::classModule($class));
        }else{
            $params['controllerConfig']['factoryClass'] = Util::classTruncated($class, Util::classModule($class));
        }

        return $params;
    }

}